package javax.swing.plaf.metal;

import java.awt.Dimension;
import javax.swing.plaf.ColorUIResource;
import java.awt.Graphics;
import javax.swing.UIManager;
import java.awt.Color;
import javax.swing.plaf.basic.BasicArrowButton;

public class MetalScrollButton extends BasicArrowButton
{
    private static Color shadowColor;
    private static Color highlightColor;
    private boolean isFreeStanding;
    private int buttonWidth;
    
    public MetalScrollButton(int direction, int buttonWidth, boolean isFreeStanding) {
        super(direction);
        this.isFreeStanding = false;
        MetalScrollButton.shadowColor = UIManager.getColor("ScrollBar.darkShadow");
        MetalScrollButton.highlightColor = UIManager.getColor("ScrollBar.highlight");
        this.buttonWidth = buttonWidth;
        this.isFreeStanding = isFreeStanding;
    }
    
    public void setFreeStanding(boolean isFreeStanding) {
        this.isFreeStanding = isFreeStanding;
    }
    
    @Override
    public void paint(Graphics graphics) {
        int leftToRight = MetalUtils.isLeftToRight(this);
        int enabled = this.getParent().isEnabled();
        ColorUIResource colorUIResource = enabled ? MetalLookAndFeel.getControlInfo() : MetalLookAndFeel.getControlDisabled();
        int pressed = this.getModel().isPressed();
        int width = this.getWidth();
        int height = this.getHeight();
        int n = (height + 1) / 4;
        if (!(pressed == 0)) {
            graphics.setColor(MetalLookAndFeel.getControlShadow());
        }
        else {
            graphics.setColor(this.getBackground());
        }
        graphics.fillRect(0, 0, width, height);
        if (this.getDirection() == 1) {
            if (!this.isFreeStanding) {
                ++height;
                graphics.translate(0, -1);
                width += 2;
                if (leftToRight == 0) {
                    graphics.translate(-1, 0);
                }
            }
            graphics.setColor(colorUIResource);
            int n8 = (height + 1 - n) / 2;
            int n9 = width / 2;
            for (int l = 0; l < n; ++l) {
                graphics.drawLine(n9 - l, n8 + l, n9 + l + 1, n8 + l);
            }
            if (!(enabled == 0)) {
                graphics.setColor(MetalScrollButton.highlightColor);
                if (pressed == 0) {
                    graphics.drawLine(1, 1, width - 3, 1);
                    graphics.drawLine(1, 1, 1, height - 1);
                }
                graphics.drawLine(width - 1, 1, width - 1, height - 1);
                graphics.setColor(MetalScrollButton.shadowColor);
                graphics.drawLine(0, 0, width - 2, 0);
                graphics.drawLine(0, 0, 0, height - 1);
                graphics.drawLine(width - 2, 2, width - 2, height - 1);
            }
            else {
                MetalUtils.drawDisabledBorder(graphics, 0, 0, width, height + 1);
            }
            if (!this.isFreeStanding) {
                --height;
                graphics.translate(0, 1);
                width -= 2;
                if (leftToRight == 0) {
                    graphics.translate(1, 0);
                }
            }
        }
        else if (this.getDirection() == 5) {
            if (!this.isFreeStanding) {
                ++height;
                width += 2;
                if (leftToRight == 0) {
                    graphics.translate(-1, 0);
                }
            }
            graphics.setColor(colorUIResource);
            int n6 = (height + 1 - n) / 2 + n - 1;
            int n7 = width / 2;
            for (int k = 0; k < n; ++k) {
                graphics.drawLine(n7 - k, n6 - k, n7 + k + 1, n6 - k);
            }
            if (!(enabled == 0)) {
                graphics.setColor(MetalScrollButton.highlightColor);
                if (pressed == 0) {
                    graphics.drawLine(1, 0, width - 3, 0);
                    graphics.drawLine(1, 0, 1, height - 3);
                }
                graphics.drawLine(1, height - 1, width - 1, height - 1);
                graphics.drawLine(width - 1, 0, width - 1, height - 1);
                graphics.setColor(MetalScrollButton.shadowColor);
                graphics.drawLine(0, 0, 0, height - 2);
                graphics.drawLine(width - 2, 0, width - 2, height - 2);
                graphics.drawLine(2, height - 2, width - 2, height - 2);
            }
            else {
                MetalUtils.drawDisabledBorder(graphics, 0, -1, width, height + 1);
            }
            if (!this.isFreeStanding) {
                --height;
                width -= 2;
                if (leftToRight == 0) {
                    graphics.translate(1, 0);
                }
            }
        }
        else if (this.getDirection() == 3) {
            if (!this.isFreeStanding) {
                height += 2;
                ++width;
            }
            graphics.setColor(colorUIResource);
            int n4 = (width + 1 - n) / 2 + n - 1;
            int n5 = height / 2;
            for (int j = 0; j < n; ++j) {
                graphics.drawLine(n4 - j, n5 - j, n4 - j, n5 + j + 1);
            }
            if (!(enabled == 0)) {
                graphics.setColor(MetalScrollButton.highlightColor);
                if (pressed == 0) {
                    graphics.drawLine(0, 1, width - 3, 1);
                    graphics.drawLine(0, 1, 0, height - 3);
                }
                graphics.drawLine(width - 1, 1, width - 1, height - 1);
                graphics.drawLine(0, height - 1, width - 1, height - 1);
                graphics.setColor(MetalScrollButton.shadowColor);
                graphics.drawLine(0, 0, width - 2, 0);
                graphics.drawLine(width - 2, 2, width - 2, height - 2);
                graphics.drawLine(0, height - 2, width - 2, height - 2);
            }
            else {
                MetalUtils.drawDisabledBorder(graphics, -1, 0, width + 1, height);
            }
            if (!this.isFreeStanding) {
                height -= 2;
                --width;
            }
        }
        else if (this.getDirection() == 7) {
            if (!this.isFreeStanding) {
                height += 2;
                ++width;
                graphics.translate(-1, 0);
            }
            graphics.setColor(colorUIResource);
            int n2 = (width + 1 - n) / 2;
            int n3 = height / 2;
            for (int i = 0; i < n; ++i) {
                graphics.drawLine(n2 + i, n3 - i, n2 + i, n3 + i + 1);
            }
            if (!(enabled == 0)) {
                graphics.setColor(MetalScrollButton.highlightColor);
                if (pressed == 0) {
                    graphics.drawLine(1, 1, width - 1, 1);
                    graphics.drawLine(1, 1, 1, height - 3);
                }
                graphics.drawLine(1, height - 1, width - 1, height - 1);
                graphics.setColor(MetalScrollButton.shadowColor);
                graphics.drawLine(0, 0, width - 1, 0);
                graphics.drawLine(0, 0, 0, height - 2);
                graphics.drawLine(2, height - 2, width - 1, height - 2);
            }
            else {
                MetalUtils.drawDisabledBorder(graphics, 0, 0, width + 1, height);
            }
            if (!this.isFreeStanding) {
                height -= 2;
                --width;
                graphics.translate(1, 0);
            }
        }
    }
    
    @Override
    public Dimension getPreferredSize() {
        if (this.getDirection() == 1) {
            return new Dimension(this.buttonWidth, this.buttonWidth - 2);
        }
        if (this.getDirection() == 5) {
            return new Dimension(this.buttonWidth, this.buttonWidth - (this.isFreeStanding ? 1 : 2));
        }
        if (this.getDirection() == 3) {
            return new Dimension(this.buttonWidth - (this.isFreeStanding ? 1 : 2), this.buttonWidth);
        }
        if (this.getDirection() == 7) {
            return new Dimension(this.buttonWidth - 2, this.buttonWidth);
        }
        return new Dimension(0, 0);
    }
    
    @Override
    public Dimension getMinimumSize() {
        return this.getPreferredSize();
    }
    
    @Override
    public Dimension getMaximumSize() {
        return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
    }
    
    public int getButtonWidth() {
        return this.buttonWidth;
    }
}
